/* tracking-base.js -- 2024/01 */
var trackingBase = {
    errorStyle: "color: #fa2020",
    successStyle: "color: #3dff33",
    customStyle: "color: #d48d00",
    typeWeb: function () {

        var anchorElements = document.body.getElementsByTagName('a');
        var trackableWebElements = [];

        /*
            Le tracking d'un élément plus web sera ignoré si celui-ci n'a pas d'attribut 'data-custom-value'.
        */
        for (var i = 0; i < anchorElements.length; i++) {

            if (anchorElements[i].getAttribute("data-custom-value") !== null) {
                var ancElm = anchorElements[i];
                trackableWebElements.push(ancElm);
            }
        }


        if (trackableWebElements.length > 0) {
            for (var x = 0; x < trackableWebElements.length; x++) {
                trackableWebElements[x].addEventListener("click", attributeValueMaker(x));
            }
        } else {
            trackingBase.throwError("no anchor element(s) found or custom attribute is missing.",
                "+web_not_found");
        }

        /*
        data-custom-value peut avoir une valeure vide, 
        dans un tel cas attributeValueMaker() créé une valeur selon l'index chonologique du plus web dans le fichier html. 
       */
        function attributeValueMaker() {

            return function () {

                var customTrackerValue = this.getAttribute("data-custom-value");

                if (customTrackerValue !== null && typeof customTrackerValue != "undefined") {

                    if (customTrackerValue != "") {
                        trackingBase.customTrack('web_' + customTrackerValue);
                    }
                    else {
                        trackingBase.customTrack('web_' + (trackableWebElements.indexOf(this) + 1));
                    }
                }
            }
        }
    },
    typeLegal: function () {

        if (typeof legal === "object") {
            var legalParent = document.querySelector('.lp-legal');

            legal.btn.addEventListener("click", function () {
                if (legalParent.classList.contains("lp-legal-active")) {
                    trackingBase.customTrack('legal_active');
                }
            });
        } else {
            trackingBase.throwError("custom tracking for legal initiated but no legal object found.",
                "legal_not_found");
        }
    },

    customTrack: function (str) {
        let customPrefix = "";
        let formattedString = "";
        str = str || "";

        /*
            prefixHelper() valide si Custom_ ou dnt_ sont présents dans le string passé en argument.
            Si les deux sont absents, il ajoute Custom_ au string.
        */
        function prefixHelper(str) {
            const regex_custom = /^custom/i;
            const regex_dnt = /^dnt/i;

            if (!regex_dnt.test(str)) {
                if (!regex_custom.test(str)) {
                    customPrefix = "custom_"
                }
            }
        }

        /*  
            Nettoyer le string custom des accents, espaces,
            caractères spéciaux et majuscules.
        */
        function cleanString(str) {
            if (!(str === null) || (typeof str == "undefined")) {
                // espaces vides
                str = str.replace(/\s/g, "_");
                // àâéèêëôöùûçï
                str = str
                    .replace(/[àâ]/g, "a")
                    .replace(/[éèêë]/g, "e")
                    .replace(/[ôö]/g, "o")
                    .replace(/[ùû]/g, "u")
                    .replace(/ç/g, "c")
                    .replace(/ï/g, "i");

                // ÀÂÉÈÊËÔÖÙÛÇÏ
                str = str
                    .replace(/[ÀÂ]/g, "A")
                    .replace(/[ÉÈÊË]/g, "E")
                    .replace(/[ÔÖ]/g, "O")
                    .replace(/[ÙÛ]/g, "U")
                    .replace(/Ç/g, "C")
                    .replace(/Ï/g, "I");

                str = customPrefix + str;
                return str;
            }
        }

        prefixHelper(str);
        formattedString = cleanString(str);

        try {
            LP.Pub.Tracking.CustomerEventPublisher.publish(formattedString);
        } catch (error) {
            console.log("%c" + formattedString, trackingBase.customStyle);
        }
    },


    callEventTracker: function (src) {

        let modifiedTagSrc;


        /*  
            Avant d'appeler un tag client on doit ajouter un préventif et lui donner un timestamp.

            /--- Remplacement du [timestamp] ---/
            Les standarts tracking veulent qu'on remplace [timestamp] (incluant les brakets) par un nombre random.
            Si [timestamp] est absent, throwError.

            Adencoder utilise l'encodage ASCII change et [timestamp] pour %255Btimestamp%255D.
            Pour la détection on doit changer '[' pour %5B et ']' pour %5D.
            Adencoder encode aussi % avec %25.
            Résultat : %255B pour remplacer %5B ('[') et %255D pour remplacer %5D (']').

            /--- Ajout du préventif 'lpprev=1' ---/
            Le préventif doit toujours être ajouté en paramètre à la fin d'un tag sinon adEncoder va le modifier/encapsuler, ce qui le rendra non fonctionel. On empêche pas l'appel du tag quand le préventif est absent car adEncoder supprime 'lpprev=1' du tag quand il est trouvé.
            
            Donc à ce stade-ci, Le trowError est seulement pour avertir ( dans la console du navigateur ) que le préventif est absent, ensuite on continu le processus d'appel du tag, qui sera fonctionnel dans le navigateur seulement.

            Dans l'application, l'absence de lpprev=1 sera prise en charge dans le onload du tag, qui va planter lors du processus d'appel.
        */

        function addTimestamp(tagSrc) {

            if (tagSrc.indexOf("[timestamp]") >= 0) {
                tagSrc = tagSrc.replace("[timestamp]", requestRandomTS());
            }
            //Au cas ou l'encodage des % n'est plus appliqué dans adEncoder.
            else if (tagSrc.indexOf("%5Btimestamp%5D") >= 0) {
                tagSrc = tagSrc.replace("%5Btimestamp%5D", requestRandomTS());
            }
            else if (tagSrc.indexOf("%255Btimestamp%255D") >= 0) {
                tagSrc = tagSrc.replace("%255Btimestamp%255D", requestRandomTS());
            }
            else {
                tagSrc = false;
            }

            function requestRandomTS() {
                var axel = Math.random() + "";
                var a = axel * 10000000000000;
                return Math.floor(a);
            }

            return tagSrc;
        }

        function lpprevChecked(tagSrc) {

            var regex = /lpprev=1/;
            var match = tagSrc.match(regex);

            if (!match) {
                return false;
            }

            return true;
        }

        function tagIsValid() {

            let tagReady = false;

            if (src != "") {

                modifiedTagSrc = addTimestamp(src);

                if (modifiedTagSrc) {
        
                    if (!lpprevChecked(modifiedTagSrc)) {
                        trackingBase.throwError("'lpprev=1' not found. Tag call will fail in app.");
                    }

                    tagReady = true;

                } else {
                    trackingBase.throwError("Event-tag failed. [timestamp] not found.", "event-tag_failed:timestamp");
                }

            } else {
                trackingBase.throwError("'Event-tag failed. 'data-client-tag' attribute is an empty string.", "event-tag_failed:no_string");
            }

            return tagReady;
        }

        function callTag(tagSrc) {
            var pixelImg = new Image();
            pixelImg.src = tagSrc;

            //onerror prend en charge l'absence de préventif ainsi que les sources non valides.
            pixelImg.onerror = function () {
                trackingBase.throwError("call to event-tag failed. Check src for errors:\n" + tagSrc, 'event-tag_falied:check_tag_src');
            }
            pixelImg.onload = function () {
                console.log("%cevent-tag:" + tagSrc, trackingBase.successStyle);
                trackingBase.customTrack('dnt_event-tag');
            }

        }

        if (tagIsValid()) {
            callTag(modifiedTagSrc);
        }
    },

    throwError: function (consoleMsg, customMsg) {

        if (customMsg != undefined) {
            trackingBase.customTrack('dnt_' + customMsg);
        }

        console.log("%c" + consoleMsg + "", trackingBase.errorStyle);

    }
};
